@@ -12,7 +12,7 @@ pub mod spec;
 use std::{
     fmt::Write,
     io::Write as IoWrite,
-    path::Path,
+    path::PathBuf,
     str::FromStr,
     sync::{
         atomic::{AtomicBool, Ordering},
@@ -23,7 +23,6 @@ use std::{
 
 use log::{log_enabled, LevelFilter};
 use monument::{Comp, Config, Progress, Query, QueryUpdate};
-use ordered_float::OrderedFloat;
 use ringing_utils::{BigNumInt, PrettyDuration};
 use simple_logger::SimpleLogger;
 use spec::Spec;
@@ -57,12 +56,12 @@ pub fn run(
     let start_time = Instant::now();
 
     // Generate & debug print the TOML file specifying the search
-    let spec = Spec::from_source(source)?;
+    let spec = Spec::from_source(&source)?;
     debug_print!(Spec, spec);
 
     // Convert the `Spec` into a `Layout` and other data required for running a search
     log::debug!("Generating query");
-    let query = spec.lower(source)?;
+    let query = spec.lower(&source)?;
     debug_print!(Query, query);
     debug_print!(Layout, &query.layout);
 
@@ -105,9 +104,10 @@ pub fn run(
         abort_flag.clone(),
     );
 
+    use ordered_float::OrderedFloat as OF;
     // Recover and sort the compositions, then return the query
     let mut comps = comps.lock().unwrap().to_vec();
-    comps.sort_by_key(|comp| OrderedFloat(comp.music_score()));
+    comps.sort_by_key(|comp| (OF(comp.music_score()), OF(comp.avg_score)));
     Ok(Some(QueryResult {
         comps,
         duration: start_time.elapsed(),
@@ -116,12 +116,13 @@ pub fn run(
 }
 
 /// The `Source` of the TOML that Monument should read.  In nearly all cases, this will be loaded
-/// from a [`Path`].  For the test runner it's useful to be able to run Monument on strings that
-/// aren't loaded from a specific file, so for this we have the [`Str`](Self::Str) variant.
-#[derive(Debug, Clone, Copy)]
+/// from the file a given [`Path`](std::path::Path).  For the test runner it's useful to be able to
+/// run Monument on strings that aren't loaded from a specific file, so for this we have the
+/// [`Str`](Self::Str) variant.
+#[derive(Debug, Clone)]
 pub enum Source<'a> {
     /// The TOML 'spec' file should be loaded from this path
-    Path(&'a Path),
+    Path(PathBuf),
     /// The TOML should be read directly from a string
     Str {
         spec: &'a str,
@@ -290,8 +291,9 @@ impl SingleLineProgressLogger {
         let p = &self.last_progress;
         write!(
             buf,
-            "    {} iters, {} items in queue, avg/max len {:.0}/{}",
+            "    {} iters, {} comps :: {} items in queue, avg/max len {:.0}/{}",
             BigNumInt(p.iter_count),
+            BigNumInt(p.num_comps),
             BigNumInt(p.queue_len),
             p.avg_length,
             p.max_length
