@@ -2,7 +2,7 @@ use std::{
     cmp::Ordering,
     collections::HashMap,
     fmt::Write,
-    ops::{Deref, Range},
+    ops::{self, Deref},
     path::{Path, PathBuf},
     time::Duration,
 };
@@ -19,11 +19,11 @@ use itertools::Itertools;
 use monument::{
     layout::{
         self,
-        new::{Call, CourseHeadMaskPreset, SpliceStyle},
+        new::{Call, CourseHeadMaskPreset},
         Layout,
     },
     music::{MusicType, StrokeSet},
-    CallType, CallVec, OptRange, Query,
+    CallType, CallVec, OptRange, Query, SpliceStyle,
 };
 use serde::Deserialize;
 
@@ -65,12 +65,15 @@ pub struct Spec {
     splice_style: SpliceStyle,
     /// Bounds on how many rows of each method is allowed
     #[serde(default)]
-    method_count: OptRange,
+    method_count: RangeInclusive,
     /// Score which is applied for every change of method.  Defaults to `0.0`
     #[serde(default)]
     splice_weight: f32,
 
     /* CALLS */
+    /// Sets the bell who's position will be used to determine calling positions.  Defaults to the
+    /// heaviest bell
+    calling_bell: Option<u8>,
     /// Which calls should be used by default
     #[serde(default)] // Default to near calls
     base_calls: BaseCalls,
@@ -91,6 +94,11 @@ pub struct Spec {
     calls: Vec<SpecificCall>,
 
     /* MUSIC */
+    /// If `default_music = true` and no other music is specified, Monument will insert a 'default'
+    /// music profile.  If you truly don't care about music, set this to `false` and don't specify
+    /// your own music.
+    #[serde(default = "get_true")]
+    default_music: bool,
     /// Path to a file containing a music definition, relative to **this** TOML file
     music_file: Option<PathBuf>,
     /// Specification of which classes of music Monument should consider
@@ -145,7 +153,7 @@ pub struct Spec {
 
 impl Spec {
     /// Read a `Spec` from a TOML file
-    pub fn from_source(source: Source) -> anyhow::Result<Self> {
+    pub fn from_source(source: &Source) -> anyhow::Result<Self> {
         let toml_buf;
         let toml_string: &str = match source {
             Source::Path(p) => {
@@ -158,7 +166,7 @@ impl Spec {
     }
 
     /// 'Lower' this `Spec`ification into a [`Query`]
-    pub fn lower(&self, source: Source) -> anyhow::Result<Query> {
+    pub fn lower(&self, source: &Source) -> anyhow::Result<Query> {
         // Lower `MethodSpec`s into `bellframe::Method`s.
         let mut loaded_methods: Vec<(bellframe::Method, MethodCommon)> = self
             .methods
@@ -178,6 +186,13 @@ impl Spec {
                     "No methods specified.  Try e.g. `method = \"Bristol Surprise Major\"`.",
                 )
             })?;
+        let calling_bell = match self.calling_bell {
+            Some(v) => Bell::from_number(v).ok_or_else(|| {
+                anyhow::Error::msg("Invalid calling bell: bell number 0 doesn't exist.")
+            })?,
+            None => stage.tenor(),
+        };
+
         // Parse the CH mask and calls, and combine these with the `bellframe::Method`s to get
         // `layout::new::Method`s
         let ch_masks = self.ch_mask_preset(stage)?.into_masks(stage);
@@ -194,7 +209,7 @@ impl Spec {
                 .iter()
                 .map(|(m, common)| (m.title(), common.shorthand.as_deref())),
         );
-        let method_count = self.method_count.or_range(&default_method_count(
+        let method_count = OptRange::from(self.method_count).or_range(&default_method_count(
             loaded_methods.len(),
             &self.length.range,
         ));
@@ -208,13 +223,15 @@ impl Spec {
                 .course_heads
                 .map(|chs| parse_ch_masks(&chs, stage))
                 .transpose()?;
-            let count_range = common.count_range.or_range(&method_count);
-            if is_spliced && common.count_range.is_set() {
+            let explicit_count_range = OptRange::from(common.count_range);
+            let count_range = explicit_count_range.or_range(&method_count);
+            if is_spliced && explicit_count_range.is_set() {
                 log::info!("count range for {:<2}: {:?}", shorthand, count_range);
             }
             methods.push(layout::new::Method::new(
                 method,
                 override_ch_masks.unwrap_or_else(|| ch_masks.clone()),
+                calling_bell,
                 shorthand,
                 count_range,
                 common.start_indices.as_deref().or(start_indices.as_deref()),
@@ -228,43 +245,56 @@ impl Spec {
         let layout = Layout::new(
             methods,
             &calls,
+            calling_bell,
             self.splice_style,
             &part_head,
             self.leadwise,
         )?;
 
-        let music_types = self.music_types(source, stage)?;
-        if music_types.is_empty() {
-            // TODO: Add default music
-            log::warn!("No music patterns specified.  Are you sure you don't care about music?");
-        }
-
         // Build this layout into a `Graph`
         Ok(Query {
             layout,
             len_range: self.length.range.clone(),
             num_comps: self.num_comps,
             allow_false: self.allow_false,
+            splice_style: self.splice_style,
 
             calls: calls.into_iter().map(CallType::from).collect(),
             part_head,
             ch_weights,
             splice_weight: self.splice_weight,
 
-            music_types,
+            music_types: self.music_types(source, stage)?,
             start_stroke: self.start_stroke,
             max_duffer_rows: self.max_duffer_rows,
         })
     }
 
-    fn music_types(&self, source: Source, stage: Stage) -> anyhow::Result<Vec<MusicType>> {
+    fn music_types(&self, source: &Source, stage: Stage) -> anyhow::Result<Vec<MusicType>> {
         // Compute the explicitly mentioned music types
         let mut music_types = self
             .music
             .iter()
             .flat_map(|spec| spec.to_music_types(stage, self.default_non_duffer))
             .collect_vec();
-        // Load the music from the `music_file`
+
+        /// Load the music from the `music_file`.  This has to be a macro, since a closure would
+        /// have to capture `&mut music_types` (thus preventing us from borrowing it again to check
+        /// `music_types.is_empty()`)
+        macro_rules! add_music_toml {
+            ($toml_str: expr) => {{
+                // Parse the TOML
+                let music_file: MusicFile = parse_toml($toml_str)?;
+                // Add the music types
+                music_types.extend(
+                    music_file
+                        .music
+                        .iter()
+                        .flat_map(|spec| spec.to_music_types(stage, self.default_non_duffer)),
+                );
+            }};
+        }
+
         if let Some(relative_music_path) = &self.music_file {
             let toml_buf;
             let music_file_toml_string: &str = match source {
@@ -286,15 +316,30 @@ impl Spec {
                 })?,
             };
 
-            // Parse the TOML
-            let music_file: MusicFile = parse_toml(music_file_toml_string)?;
-            // Add the music types
-            music_types.extend(
-                music_file
-                    .music
-                    .iter()
-                    .flat_map(|spec| spec.to_music_types(stage, self.default_non_duffer)),
-            );
+            add_music_toml!(music_file_toml_string);
+        }
+        // Add default music if `default_music = true` and there aren't any music types
+        if music_types.is_empty() && self.default_music {
+            #[rustfmt::skip]
+            let default_music_toml = match stage {
+                Stage::MINOR   => Some(include_str!("default-music-minor.toml")),
+                Stage::TRIPLES => Some(include_str!("default-music-triples.toml")),
+                Stage::MAJOR   => Some(include_str!("default-music-major.toml")),
+                Stage::ROYAL   => Some(include_str!("default-music-royal.toml")),
+                Stage::MAXIMUS => Some(include_str!("default-music-maximus.toml")),
+                _ => None,
+            };
+
+            match default_music_toml {
+                Some(toml) => {
+                    log::warn!("No music specified, so falling back on default.  Set `default_music = false` if you genuinely don't want any music scoring.");
+                    add_music_toml!(toml);
+                }
+                None => log::warn!(
+                    "No default music profile for {}.  No music will be scored.",
+                    stage
+                ),
+            }
         }
         Ok(music_types)
     }
@@ -455,19 +500,18 @@ impl Spec {
     }
 }
 
-fn parse_ch_masks(ch_mask_strings: &[String], stage: Stage) -> anyhow::Result<Vec<(Mask, Bell)>> {
+fn parse_ch_masks(ch_mask_strings: &[String], stage: Stage) -> anyhow::Result<Vec<Mask>> {
     ch_mask_strings
         .iter()
-        .map(|s| match Mask::parse_with_stage(s, stage) {
-            Ok(mask) => Ok((mask, stage.tenor())),
-            Err(e) => Err(mask_parse_error("course head mask", s, e)),
+        .map(|s| {
+            Mask::parse_with_stage(s, stage).map_err(|e| mask_parse_error("course head mask", s, e))
         })
         .collect()
 }
 
 /// Determine a suitable default range in which method counts must lie, thus enforcing decent
 /// method balance.
-fn default_method_count(num_methods: usize, len_range: &Range<usize>) -> Range<usize> {
+fn default_method_count(num_methods: usize, len_range: &ops::Range<usize>) -> ops::Range<usize> {
     let min_f32 = len_range.start as f32 / num_methods as f32 * (1.0 - METHOD_BALANCE_ALLOWANCE);
     let max_f32 = len_range.end as f32 / num_methods as f32 * (1.0 + METHOD_BALANCE_ALLOWANCE);
     // + 1 because the `method_count` is an **inclusive** range
@@ -512,7 +556,7 @@ pub struct MethodCommon {
 
     /// Optional override for method count range
     #[serde(default, rename = "count")]
-    count_range: OptRange,
+    count_range: RangeInclusive,
     /// The inputs to this map are numerical strings representing sub-lead indices, and the
     /// outputs are the lead location names
     lead_locations: Option<HashMap<String, String>>,
@@ -781,7 +825,7 @@ pub enum MusicSpec {
         pattern: String,
         /// For each pattern, which music counts are allowed
         #[serde(default)]
-        count_each: OptRange,
+        count_each: RangeInclusive,
         #[serde(flatten)]
         common: MusicCommon,
     },
@@ -789,7 +833,7 @@ pub enum MusicSpec {
         patterns: Vec<String>,
         /// For each pattern, which music counts are allowed
         #[serde(default)]
-        count_each: OptRange,
+        count_each: RangeInclusive,
         #[serde(flatten)]
         common: MusicCommon,
     },
@@ -802,7 +846,7 @@ pub struct MusicCommon {
     weight: f32,
     /// Possibly unbounded range of counts which are allowed in this music type
     #[serde(rename = "count", default)]
-    count_range: OptRange,
+    count_range: RangeInclusive,
     /// If `true`, then any chunks containing this music will be marked as 'non-duffer'
     non_duffer: Option<bool>,
     /// Which strokes this music can apply to
@@ -810,6 +854,17 @@ pub struct MusicCommon {
     strokes: StrokeSet,
 }
 
+impl Default for MusicCommon {
+    fn default() -> Self {
+        Self {
+            weight: 1.0,
+            count_range: RangeInclusive::default(),
+            non_duffer: None,
+            strokes: StrokeSet::Both,
+        }
+    }
+}
+
 impl MusicSpec {
     /// Generates a [`MusicType`] representing `self`.
     pub fn to_music_types(&self, stage: Stage, default_non_duffer: bool) -> Vec<MusicType> {
@@ -817,7 +872,7 @@ impl MusicSpec {
             /// Equivalent to [`Self::Runs`] or [`Self::RunsList`]
             Runs(&'_self [u8], &'_self bool),
             /// Equivalent to [`Self::Pattern`] or [`Self::Pattern`]
-            Patterns(&'_self [String], &'_self OptRange),
+            Patterns(&'_self [String], &'_self RangeInclusive),
         }
 
         use std::slice::from_ref;
@@ -858,15 +913,17 @@ impl MusicSpec {
                 vec![MusicType::new(
                     regexes,
                     common.weight,
-                    common.count_range,
+                    common.count_range.into(),
                     non_duffer,
                     common.strokes,
                 )]
             }
             LoweredType::Patterns(patterns, count_each) => {
                 let regexes = patterns.iter().map(|s| Regex::parse(s));
+                let count_each = OptRange::from(*count_each);
+                let count_range = OptRange::from(common.count_range);
                 if count_each.is_set() {
-                    if common.count_range.is_set() {
+                    if count_range.is_set() {
                         todo!("Mixing `count` and `count_each` isn't implemented yet!");
                     }
                     // If just `count_each` is set, we generate a separate `MusicType` for each
@@ -877,7 +934,7 @@ impl MusicSpec {
                             MusicType::new(
                                 vec![regex],
                                 common.weight,
-                                *count_each,
+                                count_each,
                                 non_duffer,
                                 common.strokes,
                             )
@@ -889,7 +946,7 @@ impl MusicSpec {
                     vec![MusicType::new(
                         regexes.collect_vec(),
                         common.weight,
-                        common.count_range,
+                        count_range,
                         non_duffer,
                         common.strokes,
                     )]
@@ -899,9 +956,40 @@ impl MusicSpec {
     }
 }
 
-//////////////////////
-// HELPER FUNCTIONS //
-//////////////////////
+/////////////
+// HELPERS //
+/////////////
+
+/// A version of [`OptRange`] which allows for convenient deserialisation from a single number
+/// (e.g. `count = 5` is equivalent to `count = { min = 5, max = 5 }`)
+#[derive(Debug, Clone, Copy, Deserialize)]
+#[serde(untagged, deny_unknown_fields)]
+pub enum RangeInclusive {
+    SingleNumber(usize),
+    Range {
+        min: Option<usize>,
+        max: Option<usize>,
+    },
+}
+
+impl Default for RangeInclusive {
+    fn default() -> Self {
+        Self::Range {
+            min: None,
+            max: None,
+        }
+    }
+}
+
+impl From<RangeInclusive> for OptRange {
+    fn from(r: RangeInclusive) -> Self {
+        let (min, max) = match r {
+            RangeInclusive::SingleNumber(n) => (Some(n), Some(n)),
+            RangeInclusive::Range { min, max } => (min, max),
+        };
+        OptRange { min, max }
+    }
+}
 
 /// Attempt to read a file as a [`String`], returning a helpful error message on failure
 fn read_file_to_string(path: &Path) -> anyhow::Result<String> {
